%--x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x--%
%-- Clculo del Kilometraje de una Carretera Real --%
%--x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x--%

% FUNCION:
%
% Realiza el kilometraje de una carretera real cuyas medidas han sido
% pasadas en un fichero de dos columnas tal que en la primera se muestran
% las medidas del eje de abcisas(X) y en la segunda las medidas del eje de
% ordenadas(Y)

% Funciones de la libreria de MATLAB utilizadas
%
%   load('nombrefichero','-ASCII') -> devuelve las variables contenidas en
%       el fichero especificado. En este caso devolvera una matriz Mx2 que
%       sern las medidas de la carretera.
%
%   size(A) -> nos devuelve las dimensiones de la matriz A. De esta forma
%       podremos averiguar el numero de medidas tomadas.
%
%   spline(X,Y) -> devuelve un polinomio 'especial' en con los coeficientes
%                  de interpolacion de los splines, a partir del cual y
%                  usando la funcion:
%
%   ppval(PP,XX) -> podremos evaluar dichos splines en los puntos indicados
%                   en XX.
%
%


% Subrutinas empleadas
%
%   longCurva :
%       Derivacion -> DerivacionNumerica
%       Integracion -> GaussLegendre   
%  


% Variables
%
% way -> Matriz que en la que se cargaran las medidas desde el fichero  
% N -> Numero de medidas tomadas. Las obtendremos con 'size'
% ESCALA -> escala de las medidas tomadas
%
% pp -> polinomio especial que contendr los coeficientes de interpolacion
%       de los splines cubicos obtenidos del trazado de la carretera con la
%       funcion spline
%
% km -> Puntos de kilometraje encontrados
% x0,x1 -> limites de integracion que estamos buscando para determinar
%           cada km
% xi -> aproximacion inicial para cada intervalo xi = x0 + 1
% u -> x1* = x1 - u
%           siendo i = integral(sqrt(1+p'^2));
%       u = (i - 1*(unKM/ESCALA))/ derivada(i);
%       uAnt = nos permitira determinar si se estan produciendo
%       oscilaciones, comparando la variacion actual con la anterior.
% error -> error permitido a la hora de determinar un KM
clear km pp x0 x1 yy u uAnt  

syms('NOMBRE', 'N', 'way', 'ESCALA', 'pp', 'x0','x1','xi','u','uAnt', 'error');
syms('L','I','J','v','y','yy', 'kk', 'unKM');

unKM = 100000; % 1km = 100.000 cm

fprintf(1,'\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
fprintf(1,'\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
fprintf(1,'\n               KILOMETRAJE DE UNA CARRETERA REAL');
fprintf(1,'\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
fprintf(1,'\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');

fprintf(1,'\n  Indique el fichero desde que se leera el trazado de la carretera: \n');
NOMBRE = input(' ','s');
fprintf(1,'\n  Indique la escala de las medidas realizadas(cm) (ej: 1cm / 0.5 km = 1/50000 :  \n');
ESCALA = input(' ');

% Cargamos los datos del trazado desde el fichero indicado

way = load(NOMBRE, '-ascii'); 
N = size(way);

% La primera columna de way ser => way(:,1) y en ella encontraremos los
% valores de X(I).
% La segunda columna de way ser => way(:,2) y en ella encontraremos los
% valores de F(X(I))


% Obtenemos los splines que interpolan al trazado en pp

%    spline( X(I)    , F(X(I)) )
pp = spline( way(:,1), way(:,2));

% Ahora debemos comenzar a buscar los puntos de kilometraje de la
% carretera.

% cm/km = 100000 / ESCALA = numero de centimetros por kilometro (en linea 
%                                                                    recta)
% Nuestro incremento bruto entre intervalos (u) va a ser de 50m en la 
%   escala real. 
% El incremento fino lo situaremos en 1m

error = 100*ESCALA; % 1m de error.

x0 = way(1,1); % Principio de carretera
km(1) = x0;
% Situamos el otro extremo de nuestra aproximacion bien lejos (1km en linea
% recta y luego iremos estrechando el cerco
xi = x0 + 1*(unKM*ESCALA);
x1 = x0;

I = 2;


while x1 <= way( N(1), 1 )
    %La expresion de la derivada con repecto a X de la longitud de la curva
    %es
        L = longCurva(pp,x0,xi);
        d = L /(xi - x0);
    u = (L - 1*(unKM*ESCALA))/d;
    
    errorL = abs(L - 1*(unKM*ESCALA));
            
    % Cuando alcancemos el error o el final de la carretera paramos
    while (errorL > error)
            x1= xi - u;
            xi = x1;
        
         % Actualizamos el valor de u
            L = longCurva(pp,x0,xi);
            d = L /(xi - x0);
         uAnt = u;   
         u = (L - 1*(unKM*ESCALA))/d;
         
         % Para evitar oscilaciones en el caso de grandes pendientes
         if(abs(u) >= abs(uAnt))
             u = sign(u)*(abs(u) - abs(uAnt))/3;
         end
       
         errorL = abs(L - 1*(unKM*ESCALA));
    end
    %Por si la primera aproximacion ha sido la buena   
    x1 = xi;
    
    %Para cuando finalicemos y nos hayamos salido del trazado
    if (x1 <= way(N(1),1))
        km(I) = x1;
    
        x0 = x1;
        xi = x0 + 1*(unKM*ESCALA);
        I = I + 1;
    end    
    

end 

    
%Ahora dibujaremos la curva y sus puntos de kilometraje

 yy = zeros(1,101*(N(1)-1));
 v = zeros(1,101*(N(1)-1));
 
 for J = 1:N(1)-1    
 for I = 1:101
    v(101*(J-1)+I)=way(J,1) + (I-1)*(way(J+1,1)-way(J,1))/100;    
 end
 end

 % Evaluacion de la funcion para dibujar el trazado
 yy = ppval(pp,v);
 
 % Obtenemos la evaluacion de los puntos kilometricos
 
 kk = ppval(pp,km);
 
 %Calculamos los km totales del trazado
 kmTotal = size(km);
 kmTotal = kmTotal(2) - 1;
 kmTotal = kmTotal + longCurva(pp,km(kmTotal + 1), way(N(1),1));

 % Creamos la grafica e introducimos los datos obtenidos
 figure(1);
 plot(v(:),yy(:),'-r',km(:), kk(:), 'b+')
 title(sprintf('Trazado de la carretera. KM TOTALES: %3.4f',kmTotal));
 xlabel(sprintf('Km. ( Escala : 1 CM : %i KM  )',1/(ESCALA*unKM)));
 ylabel('Km');
 
 fprintf(1,'\n\n Los puntos kilomtricos los podemos localizar en las siguientes coordenadas de abcisas\n\n');
 for I= 1:ceil(kmTotal)
    fprintf(1,' KM %3i  ->  X = %f \n', I - 1, km(I));
 end
    fprintf(1,'\n     Fin de Trazado en KM = %3.4f\n', kmTotal);


 







